@@ -25,7 +25,8 @@ class _RNNModule(nn.Module):
                  input_size: int,
                  hidden_dim: int,
                  num_layers: int,
-                 target_size: int = 1,
+                 target_size: int,
+                 nr_params: int,
                  dropout: float = 0.):
 
         """ PyTorch module implementing an RNN to be used in `RNNModel`.
@@ -47,11 +48,10 @@ class _RNNModule(nn.Module):
             The number of recurrent layers.
         target_size
             The dimensionality of the output time series.
+        nr_params
+            The number of parameters of the likelihood (or 1 if no likelihood is used).
         dropout
             The fraction of neurons that are dropped in all-but-last RNN layers.
-        likelihood
-            Optionally, the likelihood model to be used for probabilistic forecasts.
-            Expects an instance of 'darts.utils.likelihood_model.Likelihood'.
 
         Inputs
         ------
@@ -60,7 +60,7 @@ class _RNNModule(nn.Module):
 
         Outputs
         -------
-        y of shape `(batch_size, input_length, output_size)`
+        y of shape `(batch_size, output_chunk_length, target_size, nr_params)`
             Tensor containing the outputs of the RNN at every time step of the input sequence.
             During training the whole tensor is used as output, whereas during prediction we only use y[:, -1, :].
             However, this module always returns the whole Tensor.
@@ -70,13 +70,14 @@ class _RNNModule(nn.Module):
 
         # Defining parameters
         self.target_size = target_size
+        self.nr_params = nr_params
         self.name = name
 
         # Defining the RNN module
         self.rnn = getattr(nn, name)(input_size, hidden_dim, num_layers, batch_first=True, dropout=dropout)
 
         # The RNN module needs a linear layer V that transforms hidden states into outputs, individually
-        self.V = nn.Linear(hidden_dim, target_size)
+        self.V = nn.Linear(hidden_dim, target_size * nr_params)
 
     def forward(self, x, h=None):
         # data is of size (batch_size, input_length, input_size)
@@ -89,7 +90,7 @@ class _RNNModule(nn.Module):
         predictions = self.V(out)
 
         # predictions is of size (batch_size, input_length, target_size)
-        predictions = predictions.view(batch_size, -1, self.target_size)
+        predictions = predictions.view(batch_size, -1, self.target_size, self.nr_params)
 
         # returns outputs for all inputs, only the last one is needed for prediction time
         return predictions, last_hidden_state
@@ -227,21 +228,21 @@ class RNNModel(TorchParametricProbabilisticForecastingModel, DualCovariatesTorch
         # historic_future_covariates and future_covariates have the same width
         input_dim = train_sample[0].shape[1] + (train_sample[1].shape[1] if train_sample[1] is not None else 0)
         output_dim = train_sample[-1].shape[1]
+        nr_params = 1 if self.likelihood is None else self.likelihood.num_parameters
 
-        target_size = (
-            self.likelihood.num_parameters * output_dim if self.likelihood is not None else output_dim
-        )
         if self.rnn_type_or_module in ['RNN', 'LSTM', 'GRU']:
             model = _RNNModule(name=self.rnn_type_or_module,
                                input_size=input_dim,
-                               target_size=target_size,
+                               target_size=output_dim,
+                               nr_params=nr_params,
                                hidden_dim=self.hidden_dim,
                                dropout=self.dropout,
                                num_layers=self.n_rnn_layers)
         else:
             model = self.rnn_type_or_module(name='custom_module',
                                             input_size=input_dim,
-                                            target_size=target_size,
+                                            target_size=output_dim,
+                                            nr_params=nr_params,
                                             hidden_dim=self.hidden_dim,
                                             dropout=self.dropout,
                                             num_layers=self.n_rnn_layers)
@@ -272,11 +273,11 @@ class RNNModel(TorchParametricProbabilisticForecastingModel, DualCovariatesTorch
 
     @random_method
     def _produce_predict_output(self, x, last_hidden_state=None):
+        output, hidden = self.model(x, last_hidden_state)
         if self.likelihood:
-            output, hidden = self.model(x, last_hidden_state)
             return self.likelihood.sample(output), hidden
         else:
-            return self.model(x, last_hidden_state)
+            return output.squeeze(dim=-1), hidden
 
     def _get_batch_prediction(self, n: int, input_batch: Tuple, roll_size: int) -> torch.Tensor:
         """
