@@ -7,12 +7,14 @@ from typing import Union, Sequence, Optional, Tuple
 import numpy as np
 
 from ...timeseries import TimeSeries
-from .training_dataset import (PastCovariatesTrainingDataset,
+from .training_dataset import (TrainingDataset,
+                               PastCovariatesTrainingDataset,
                                FutureCovariatesTrainingDataset,
                                DualCovariatesTrainingDataset,
                                MixedCovariatesTrainingDataset,
-                               SplitCovariatesTrainingDataset)
-from .utils import _get_matching_index
+                               SplitCovariatesTrainingDataset,
+                               CovariateType)
+
 from ..utils import raise_if_not
 
 
@@ -379,7 +381,7 @@ class SplitCovariatesShiftedDataset(SplitCovariatesTrainingDataset):
         return past_target, past_covariate, future_covariate, future_target
 
 
-class GenericShiftedDataset:
+class GenericShiftedDataset(TrainingDataset):
     def __init__(self,
                  target_series: Union[TimeSeries, Sequence[TimeSeries]],
                  covariates: Optional[Union[TimeSeries, Sequence[TimeSeries]]] = None,
@@ -459,59 +461,44 @@ class GenericShiftedDataset:
                      '`max(self.input_chunk_length, self.shift + self.output_chunk_length)` '
                      '({}-th series)'.format(ts_idx))
 
-        # Determine the index of the end of the output, starting from the end.
-        # It is originally in [0, self.max_samples_per_ts), so we use a modulo to have it in [0, n_samples_in_ts)
-        end_of_output_idx = (idx - (ts_idx * self.max_samples_per_ts)) % n_samples_in_ts
+        # determine the index at the end of the output chunk
+        # it is originally in [0, self.max_samples_per_ts), so we use a modulo to have it in [0, n_samples_in_ts)
+        end_of_output_idx = len(ts_target) - (idx - (ts_idx * self.max_samples_per_ts)) % n_samples_in_ts
 
-        # select forecast point and target period, using the previously computed indexes
-        if end_of_output_idx == 0:
-            # we need this case because "-0" is not supported as an indexing bound
-            future_target = target_vals[-self.output_chunk_length:]
-        else:
-            future_target = target_vals[-(self.output_chunk_length + end_of_output_idx):-end_of_output_idx]
+        # optionally, load covariates
+        ts_covariate = self.covariates[ts_idx] if self.covariates is not None else None
+        cov_type = CovariateType.NONE
+        if self.covariates is not None:
+            cov_type = CovariateType.FUTURE if self.shift_covariates else CovariateType.PAST
 
-        # starting from the end
-        start_of_input_idx = end_of_output_idx + self.output_chunk_length + self.shift
+        # get all indices for the current sample
+        past_start, past_end, future_start, future_end, cov_start, cov_end = \
+            self._memory_indexer(ts_idx=ts_idx,
+                                 ts_target=ts_target,
+                                 shift=self.shift,
+                                 input_chunk_length=self.input_chunk_length,
+                                 output_chunk_length=self.output_chunk_length,
+                                 end_of_output_idx=end_of_output_idx,
+                                 ts_covariate=ts_covariate,
+                                 cov_type=cov_type)
 
-        # select input period; look at the `input_chunk_length` points before the forecast point
-        if -(start_of_input_idx - self.input_chunk_length) == 0:
-            # handle "-0" indexing bound
-            past_target = target_vals[-start_of_input_idx:]
-        else:
-            past_target = target_vals[-start_of_input_idx:-(start_of_input_idx - self.input_chunk_length)]
+        # extract sample target
+        future_target = target_vals[future_start:future_end]
+        past_target = target_vals[past_start:past_end]
 
-        # optionally also produce the input covariate
+        # optionally, extract sample covariates
         covariate = None
         if self.covariates is not None:
-            ts_covariate = self.covariates[ts_idx]
-            cov_vals = ts_covariate.values(copy=False)
+            raise_if_not(cov_end <= len(ts_covariate),
+                         f"The dataset contains {cov_type.value} covariates "
+                         f"that don't extend far enough into the future. ({idx}-th sample)")
 
-            if self.shift_covariates:
-                # We need to return the future covariates. In this case we use the same indexing as for
-                # "future_target" (shifting the index if the time axes of target and covariate are not the same)
-                end_of_output_idx_cov = _get_matching_index(ts_target, ts_covariate, end_of_output_idx)
-                raise_if_not(end_of_output_idx_cov >= 0,
-                             "The dataset contains some covariates that don't extend far enough into the future. "
-                             "({}-th sample)".format(idx))
-                if end_of_output_idx_cov == 0:
-                    # we need this case because "-0" is not supported as an indexing bound
-                    covariate = cov_vals[-self.output_chunk_length:]
-                else:
-                    covariate = cov_vals[-(self.output_chunk_length + end_of_output_idx_cov):-end_of_output_idx_cov]
-
-            else:
-                # We need to return the past covariates. In this case we use the same indexing as for
-                # "past_target" (shifting the index if the time axes of target and covariate are not the same)
-                start_of_input_idx_cov = _get_matching_index(ts_target, ts_covariate, start_of_input_idx)
-                if -(start_of_input_idx_cov - self.input_chunk_length) == 0:
-                    # handle "-0" indexing bound
-                    covariate = cov_vals[-start_of_input_idx_cov:]
-                else:
-                    covariate = cov_vals[-start_of_input_idx_cov:-(start_of_input_idx_cov - self.input_chunk_length)]
+            covariate = ts_covariate.values(copy=False)[cov_start:cov_end]
 
             raise_if_not(len(covariate) == (self.output_chunk_length if self.shift_covariates else
                                             self.input_chunk_length),
-                         "The dataset contains some covariate series whose time axis doesn't allow to "
-                         "obtain the input (or output) chunk relative to the target series.")
+                         f"The dataset contains {cov_type.value} covariates "
+                         f"whose time axis doesn't allow to obtain the input (or output) chunk relative to the "
+                         f"target series.")
 
         return past_target, covariate, future_target
